@@ -2,9 +2,9 @@
    drvwmf.cpp : This file is part of pstoedit
 	 Backend for Windows Meta File (WMF/EMF)
 
-   Copyright (C) 1996,1997 Jens Weber, rz47b7@PostAG.DE
+   Copyright (C) 1996,1997 Jens Weber, rz47b7_AT_PostAG.DE
    Copyright (C) 1998 Thorsten Behrens and Bjoern Petersen
-   Copyright (C) 1998 - 2003 Wolfgang Glunz
+   Copyright (C) 1998 - 2005 Wolfgang Glunz
    Copyright (C) 2000 Thorsten Behrens
 
     This program is free software; you can redistribute it and/or modify
@@ -81,20 +81,11 @@ struct PLACEABLEHEADER {
 	WORD checksum;
 };
 
-
-static const OptionDescription driveroptions[] = {
-	OptionDescription("-m",0,"map to Arial"),
-	OptionDescription("-nf",0,"emulate narrow fonts"),
-	OptionDescription("-b",0,"draw bounding box"),
-	OptionDescription("-p",0,"prune line ends"),
-	OptionDescription("-nb",0,"dont calculate and write bounding box"),
-	endofoptions};
-
-static	const char description[] = "generated by WMF/EMF backend of pstoedit\0input\0\0";
+static const char description[] = "generated by WMF/EMF backend of pstoedit\0input\0\0";
 
 drvWMF::derivedConstructor(drvWMF):
-constructBase, enhanced(false), mapToArial(false),
-emulateNarrowFonts(false), drawBoundingBox(true), pruneLineEnds(false), narrowbox(true)
+constructBase, enhanced(false)
+ //drawBoundingBox(true), narrowbox(true)
 {
 	// do some consistency checking
 	if (sizeof(PLACEABLEHEADER) != PLACEABLESIZE) {
@@ -110,12 +101,12 @@ emulateNarrowFonts(false), drawBoundingBox(true), pruneLineEnds(false), narrowbo
 	}
 
 	desktopDC = GetDC(GetDesktopWindow());
-//wogl	tempName = NIL;
 
 	// which output format?
 	if (strcmp(Pdriverdesc->symbolicname, "emf") == 0) {
 		enhanced = true;
 	}
+#if 0
 	// any driver options?
 	if (driveroptions_p != NIL) {
 		// map everything to arial?
@@ -138,13 +129,14 @@ emulateNarrowFonts(false), drawBoundingBox(true), pruneLineEnds(false), narrowbo
 			pruneLineEnds = true;
 		}
 	}
+#endif
 
 	if (enhanced) {
 		// enhanced-Metafile (memory based) for W95/98 or NT
-		if (narrowbox) {
+		if (options->narrowbox) {
 			metaDC = CreateEnhMetaFile(desktopDC, NIL, NIL, NIL);
 		} else {
-	// under non Windowssystems we cannot use PlayEnhMetafile
+	// under non Windows systems we cannot use PlayEnhMetafile
 			metaDC = CreateEnhMetaFile(desktopDC, outFileName.value(), NIL, description);
 		}
 		if (!metaDC) {
@@ -198,7 +190,7 @@ emulateNarrowFonts(false), drawBoundingBox(true), pruneLineEnds(false), narrowbo
 	coloredBrush = 0L;
 
 	// set default font
-	if (mapToArial) {
+	if (options->mapToArial) {
 		const char *const defaultfontname = "Arial";
 		setCurrentFontName(defaultfontname, false /* is standard font */ );
 	} else {
@@ -230,7 +222,7 @@ drvWMF::~drvWMF()
 	}
 	// set dummy pixel, if not EMF
 //  if( !enhanced && drawBoundingBox )
-	if (drawBoundingBox) {
+	if (options->drawBoundingBox) {
 		// standard metafile does not care much about user space.
 		// so we have to draw two white dummy points at the pic's boundaries
 		SetPixel(metaDC, minX, minY, RGB(255, 255, 255));
@@ -287,7 +279,7 @@ drvWMF::~drvWMF()
 
 		// create our final metafile (metaDC is reused)
 		// need to have two metafiles, because we know the output dimension just now
-		if (narrowbox) {
+		if (options->narrowbox) {
 			metaDC = CreateEnhMetaFile(desktopDC, outFileName.value(), NIL, description);
 		}
 		if (metaDC) {
@@ -312,7 +304,7 @@ drvWMF::~drvWMF()
 //          const RECT  bbox = {minX, maxY, maxX-minX, maxY-minY};
 //          const RECT  bbox = {minX, maxY, maxX+2*minX, -minY-maxY};
 
-			if (narrowbox) {
+			if (options->narrowbox) {
 			// replay recorded metafile
 				if (!PlayEnhMetaFile(metaDC, hMeta, &bbox)) {
 					errf << "ERROR: cannot replay created metafile" << endl;
@@ -378,8 +370,8 @@ drvWMF::~drvWMF()
 			fclose(inFile);
 		}
 		remove(tempName.value());
-//wogl		free(tempName);
-//		tempName = NIL;
+		// cout << "tmp name " << tempName.value() << endl;
+
 	}
 
 	// delete desktop DC (might need it above)
@@ -468,7 +460,7 @@ int drvWMF::fetchFont(const TextInfo & textinfo, short int textHeight, short int
 	if (strstr(textinfo.currentFontWeight.value(), "Normal"))
 		theFontRec.lfWeight = FW_NORMAL;
 
-	if (emulateNarrowFonts) {
+	if (options->emulateNarrowFonts) {
 		if (strstr(textinfo.currentFontWeight.value(), "Thin") ||
 			strstr(textinfo.currentFontName.value(), "Thin") ||
 			strstr(textinfo.currentFontFullName.value(), "Thin")) {
@@ -552,13 +544,20 @@ int drvWMF::fetchFont(const TextInfo & textinfo, short int textHeight, short int
 		(strstr(textinfo.currentFontFullName.value(), "symbol") != NIL)) {
 		theFontRec.lfCharSet = SYMBOL_CHARSET;
 		strcpy(theFontRec.lfFaceName, "symbol");
-	} else {
+	} else if ((strstr(textinfo.currentFontFamilyName.value(), "Computer Modern") != NIL) ) {
+		// special handling for TeX Fonts - fix supplied by James F. O'Brien (job at cs.berkeley.edu)
+        theFontRec.lfWeight = FW_NORMAL;
+  	    theFontRec.lfItalic = 0;
+  	    theFontRec.lfUnderline = 0;
+	    theFontRec.lfCharSet = ANSI_CHARSET;
+  	    strcpy(theFontRec.lfFaceName,  textinfo.currentFontName.value());
+	} else 	{
 		theFontRec.lfCharSet = ANSI_CHARSET;
 
-		if (mapToArial)
+		if (options->mapToArial)
 			strcpy(theFontRec.lfFaceName, "Arial");
-		else
-			strcpy(theFontRec.lfFaceName, textinfo.currentFontFamilyName.value());
+		else /* formerly we used currentFontFamilyName but FontName seems to be better */
+			strcpy(theFontRec.lfFaceName, textinfo.currentFontName.value());
 	}
 
 	if (myFont) {
@@ -588,9 +587,9 @@ void drvWMF::drawPoly(POINT * aptlPoints, int *aptlNumPts, polyType type)
 	unsigned int numOfPts = 0;
 	unsigned int p = 0;
 
-
-	for (unsigned int n = 0; n < numberOfElementsInPath() &&
-		 p < 2 * numberOfElementsInPath() && numOfPolies < 2 * numberOfElementsInPath(); n++) {
+	const unsigned int numOfElements = numberOfElementsInPath();
+	for (unsigned int n = 0; n < numOfElements &&
+		 p < 2 * numberOfElementsInPath() && numOfPolies < 2 * numOfElements; n++) {
 		const basedrawingelement & elem = pathElement(n);
 
 		if (elem.getType() != closepath) {
@@ -853,7 +852,7 @@ void drvWMF::show_path()
 
 	delete[]aptlPoints;
 	delete[]aptlNumPts;
-};
+}
 
 
 void drvWMF::show_text(const TextInfo & textinfo)
@@ -915,8 +914,8 @@ void drvWMF::show_text(const TextInfo & textinfo)
 		maxStatus = 1;
 	}
 
-	long textLen = strlen(textinfo.thetext.value());
-	if (pruneLineEnds) {
+	unsigned int textLen = strlen(textinfo.thetext.value());
+	if (options->pruneLineEnds) {
 		/* check for '#' at lineend */
 		if (textLen > 0 && textinfo.thetext.value()[textLen - 1] == '#') {
 			/* write one character less */
@@ -924,12 +923,78 @@ void drvWMF::show_text(const TextInfo & textinfo)
 		}
 	}
 
+#if defined(_WIN32)
 	TextOut(metaDC, x1, y1, textinfo.thetext.value(), textLen);
+	// TextOut(metaDC, 10, 10, "hello",4);
+#else
+
+	if (options->notforWindows) {
+		// the user is aware of generating non-portable output
+		TextOut(metaDC, x1, y1, textinfo.thetext.value(), textLen);
+	} else {
+		// we are not running Windows - so we need an emulation - see note below
+		const long textdistance = (long) sqrt((double)((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)));
+		const int letterspace = (textLen > 1) ? (textdistance / (textLen-1)) : 0 ;
+		// if there is just one char in the text, then the inter letter spacing is 0 anyway
+		int * pxDistance = new int[textLen];
+		for (unsigned int letter = 0; letter < textLen; letter++) {
+			pxDistance[letter] = letterspace;
+		}
+		const UINT fuOptions = 0;
+		ExtTextOut (metaDC, x1, y1, fuOptions, 0, textinfo.thetext.value(), textLen, pxDistance);
+		delete [] pxDistance;
+
+		static bool warningwritten = false;
+		if (textLen > 1 && !warningwritten) {
+			warningwritten = true;
+			errf << "Warning: Inter letter spacing is approximated by pstoedit because of problems in libemf. Use -pta option if results are not OK." << endl;
+		}
+	}
+	
+#endif
+
+#if 0
+
+ExtTextOut (hdc, xStart, yStart, iOptions, &rect, pString, iCount, pxDistance) ;
+The fifth argument is a pointer to a rectangle structure. This is either a 
+clipping rectangle, (if iOptions is set to ETO_CLIPPED, or a background 
+rectangle to be filled with the current background color, 
+if iOptions is set to ETO_OPAQUE. You can specify both options or neither. 
+
+The last argument is an array of integers that specify the spacing between 
+consecutive characters in the string. This allows a program to tighten or 
+loosen intercharacter spacing, which is sometimes required for justifying 
+a single word of text in a narrow column. 
+The argument can be set to NULL for default character spacing. 
+
+Note by Glunz: Even if the spacing argument can be set to NULL according to 
+the description above, this creates a problem when libemf is used.
+Libemf passes this NULL to the generated EMF file which then causes the
+file to be no longer usable under newer versions of Windows.
+
+It seems so that the Windows-API calculates this inter letter spacing automatically
+before writing the record to the EMF file. But this needs access to font information
+which is not easily done in libemf. 
+Hence one work around is to assume a fixed width font and approximate the 
+inter letter spacing by dividing the distance (P_end - P_start) by 
+the number of characters in the string.
+
+#endif
+
+
+
+
 }
 
 
 void drvWMF::show_rectangle(const float llx, const float lly, const float urx, const float ury)
 {
+	if (1) {
+		show_path();
+	} else {
+// this code is disabled - see note below.
+// needs to be fixed. 
+
 	RECT localRect;
 
 	// update pen/brush
@@ -971,11 +1036,16 @@ void drvWMF::show_rectangle(const float llx, const float lly, const float urx, c
 		maxStatus = 1;
 	}
 
-	if (currentShowType() == drvbase::stroke) {
+	if (0 && currentShowType() == drvbase::stroke) {
+		// wogl - this code is disabled. I don't know why this was this way. 
+		// one cannot use a RECT as Point * and a polyline needs 4 points to make a RECT.
+		// strange ....
 		Polyline(metaDC, (POINT *) & localRect, 2);
+		// but also using a Rectangle isn't correct. 
 	} else {
 		Rectangle(metaDC, l_transX(llx), l_transY(lly), l_transX(urx), l_transY(ury));
 	}
+	}
 }
 
 
@@ -989,10 +1059,10 @@ void drvWMF::show_image(const PSImage & image)
 	// but also transformation matrix!
 
 	// scale bounding box
-	lowerLeft.x_ *= scale;
-	lowerLeft.y_ *= scale;
-	upperRight.x_ *= scale;
-	upperRight.y_ *= scale;
+	lowerLeft.x_ *= getScale();
+	lowerLeft.y_ *= getScale();
+	upperRight.x_ *= getScale();
+	upperRight.y_ *= getScale();
 
 	const long width  = abs(i_transX(upperRight.x_) - i_transX(lowerLeft.x_));
 	const long height = abs(i_transY(upperRight.y_) - i_transY(lowerLeft.y_));
@@ -1051,10 +1121,10 @@ void drvWMF::show_image(const PSImage & image)
 							image.normalizedImageCurrentMatrix[2] *
 							image.normalizedImageCurrentMatrix[1]);
 	const float inverseMatrix[] = {
-		image.normalizedImageCurrentMatrix[3] / matrixScale / scale,
-		-image.normalizedImageCurrentMatrix[1] / matrixScale / scale,
-		-image.normalizedImageCurrentMatrix[2] / matrixScale / scale,
-		image.normalizedImageCurrentMatrix[0] / matrixScale / scale,
+		image.normalizedImageCurrentMatrix[3] / matrixScale / getScale(),
+		-image.normalizedImageCurrentMatrix[1] / matrixScale / getScale(),
+		-image.normalizedImageCurrentMatrix[2] / matrixScale / getScale(),
+		image.normalizedImageCurrentMatrix[0] / matrixScale / getScale(),
 		(image.normalizedImageCurrentMatrix[2] *
 		 image.normalizedImageCurrentMatrix[5] -
 		 image.normalizedImageCurrentMatrix[4] *
@@ -1160,7 +1230,13 @@ void drvWMF::show_image(const PSImage & image)
 
 
 
-static DriverDescriptionT < drvWMF > D_wmf("wmf", "Windows metafile", "wmf", true,	// backend supports subpathes
+#if defined(_WIN32)
+//
+// Wmf works only under Windows since the libemf has only a broken emulation of wmf.
+// Under libemf, a createMetaFile effectively creates an enhMetaFile but that confuses almost all
+// programs which expect to read an WMF with an aldus placeable header
+// 
+static DriverDescriptionT < drvWMF > D_wmf("wmf", "Windows metafile", "","wmf", true,	// backend supports subpathes
 										   // if subpathes are supported, the backend must deal with
 										   // sequences of the following form
 										   // moveto (start of subpath)
@@ -1179,10 +1255,11 @@ static DriverDescriptionT < drvWMF > D_wmf("wmf", "Windows metafile", "wmf", tru
 										   DriverDescription::memoryeps,	// no support for PNG file images
 										   DriverDescription::noopen,	// we open output file ourselves
 										   false,	// if format supports multiple pages in one file (DEFINETELY not) 
-										   false, /*clipping */ 
-										   driveroptions);
+										   false  /*clipping */ 
+										   );
+#endif
 
-static DriverDescriptionT < drvWMF > D_emf("emf", "Enhanced Windows metafile", "emf", true,	// backend supports subpathes
+static DriverDescriptionT < drvWMF > D_emf("emf", "Enhanced Windows metafile", "","emf", true,	// backend supports subpathes
 										   // if subpathes are supported, the backend must deal with
 										   // sequences of the following form
 										   // moveto (start of subpath)
@@ -1201,7 +1278,6 @@ static DriverDescriptionT < drvWMF > D_emf("emf", "Enhanced Windows metafile", "
 										   DriverDescription::memoryeps,	// no support for PNG file images
 										   DriverDescription::noopen,	// we open output file ourselves
 										   false,	// if format supports multiple pages in one file (DEFINETELY not) 
-										   false, /*clipping */
-										   driveroptions);
- 
+										   false  /*clipping */
+										   );
  
