@@ -2,7 +2,7 @@
    dynload.h : This file is part of pstoedit
    declarations for dynamic loading of drivers
 
-   Copyright (C) 1993 - 2003 Wolfgang Glunz, wglunz@pstoedit.net
+   Copyright (C) 1993 - 2005 Wolfgang Glunz, wglunz34_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,11 +20,11 @@
 
 */
 #include "cppcomp.h"
-
+#include I_iostream
 
 // we need __linux__ instead of just linux since the latter is not defined when -ansi is used.
 
-#if defined(__sparc) || defined(__linux) || defined(__linux__) || defined(__FreeBSD__) || defined(_WIN32) || defined(__OS2__) || (HAVE_DLFCN_H==1)
+#if defined(__sparc) || defined(__linux) || defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(_WIN32) || defined(__OS2__) || defined(_AIX) || (defined (HAVE_DLFCN_H) && (HAVE_DLFCN_H==1 ) ) 
 #define HAVESHAREDLIBS
 #endif
 
@@ -39,17 +39,14 @@
 #include I_stdio
 #include I_stdlib
 #include I_string_h
-#include I_iostream
 
 // for DriverDescription types.
 
+#if defined(__hpux) && defined (__GNUG__)
 typedef void (*initfunctype) ();
-#if defined(__linux) || defined(__linux__) 
-#include <dlfcn.h>
-//  static const char * const libsuffix = ".so";
-#elif defined(__FreeBSD__)
-#include <dlfcn.h>
-#elif defined(__hpux)
+#endif
+
+#if defined(__linux) || defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__hpux) || defined(_AIX) || (defined (HAVE_DLFCN_H) && (HAVE_DLFCN_H==1 ) ) 
 #include <dlfcn.h>
 #elif defined(__sparc)
 #if defined(__SVR4)
@@ -64,11 +61,11 @@ extern "C" {
 }
 #define RTLD_LAZY       1
 #endif
-// typedef void (*initfunctype) ();
+
 #elif defined(__OS2__)
 #include <sys/types.h>
 #include <dlfcn.h>
-// typedef void (*initfunctype) ();
+
 #elif defined(_WIN32)
 static char dlerror()
 {
@@ -85,9 +82,10 @@ static char dlerror()
 
 DynLoader::DynLoader(const char *libname_P, int verbose_p):libname(libname_P), handle(0), verbose(verbose_p)
 {
-	if (verbose) cerr << "dlopening " << libname << endl;
-	if (libname)
+	if (libname) {
+		if (verbose) cerr << "dlopening " << libname << endl;
 		open(libname);
+	}
 }
 
 void DynLoader::open(const char *libname_P )
@@ -100,25 +98,13 @@ void DynLoader::open(const char *libname_P )
 	strcpy(fulllibname, libname_P);
 	// not needed strcat(fulllibname_P,libsuffix);
 
-#if defined(__linux) || defined(__linux__) 
-	int loadmode = RTLD_LAZY;	// RTLD_NOW
-	handle = dlopen(fulllibname, loadmode);
-#elif defined(__FreeBSD__)
-    	int loadmode = RTLD_LAZY;       // RTLD_NOW
-   	 handle = dlopen(fulllibname, loadmode);
-#elif defined(__hpux)
-	int loadmode = RTLD_LAZY;	// RTLD_NOW
-	handle = dlopen(fulllibname, loadmode);
-#elif defined(__sparc)
-	int loadmode = RTLD_LAZY;	// RTLD_NOW
-	handle = dlopen(fulllibname, loadmode);
-#elif defined(__OS2__)
+#if defined(__linux) || defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__hpux) || defined(__sparc) || defined(__OS2__) || defined(_AIX) || (defined (HAVE_DLFCN_H) && (HAVE_DLFCN_H==1 ) ) 
 	int loadmode = RTLD_LAZY;	// RTLD_NOW
 	handle = dlopen(fulllibname, loadmode);
 #elif defined(_WIN32)
 	handle = WINLOADLIB(fulllibname);
 #else
-	system unsupported so far
+	#error "system unsupported so far"
 #endif
 	if (handle == 0) {
 		cerr << "Problem during opening " << fulllibname << ":" << dlerror()
@@ -127,10 +113,10 @@ void DynLoader::open(const char *libname_P )
 		return;
 	}
 	{
-	if (verbose) cerr << "loading dynamic library " << fulllibname << " completed successfully" << endl;
+		if (verbose) cerr << "loading dynamic library " << fulllibname << " completed successfully" << endl;
 #if defined(__hpux) && defined (__GNUG__)
-	// in HPUX g++ this is called "_GLOBAL__DI" - on other systems it may be something like "_init"
-	// under HP-UX automatic initialization of dlopened shared libs doesn't work - at least not when compiled with g++
+		// in HPUX g++ this is called "_GLOBAL__DI" - on other systems it may be something like "_init"
+		// under HP-UX automatic initialization of dlopened shared libs doesn't work - at least not when compiled with g++
 		initfunctype fptr = (initfunctype) dlsym(handle, "_GLOBAL__DI");
 		if (fptr) {
 			if (verbose) cerr << "Found an init function for the library, so execute it " << endl;
@@ -138,26 +124,18 @@ void DynLoader::open(const char *libname_P )
 		}
 #endif
 	}
-	delete[]fulllibname;
+	delete [] fulllibname;
 }
 
 void DynLoader::close()
 {
 	if (handle) {
-#if defined(__linux) || defined(__linux__) 
-		dlclose(handle);
-#elif defined(__FreeBSD__)
-        dlclose(handle);
-#elif defined(__hpux)
-		dlclose(handle);
-#elif defined(__sparc)
-		dlclose(handle);
-#elif defined(__OS2__)
+#if defined(__linux) || defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__hpux) || defined(__sparc) || defined(__OS2__) || defined(_AIX) || (defined (HAVE_DLFCN_H) && (HAVE_DLFCN_H==1 ) ) 
 		dlclose(handle);
 #elif defined(_WIN32)
-		WINFREELIB((HINSTANCE) handle);
+		(void)WINFREELIB((HINSTANCE) handle);
 #elif
-		system unsupported so far
+		#error "system unsupported so far"
 #endif
 			handle = 0;
 	}
@@ -175,20 +153,12 @@ int DynLoader::knownSymbol(const char *name) const
 
 DynLoader::fptr DynLoader::getSymbol(const char *name, int check) const
 {
-#if defined(__linux) || defined(__linux__) 
-	DynLoader::fptr rfptr = (DynLoader::fptr) dlsym(handle, name);	//lint !e611 //: Suspicious cast
-#elif defined(__FreeBSD__)
-    DynLoader::fptr rfptr = (DynLoader::fptr) dlsym(handle, name);  //lint !e611 //: Suspicious cast
-#elif defined(__hpux)
-	DynLoader::fptr rfptr = (DynLoader::fptr) dlsym(handle, name);	//lint !e611 //: Suspicious cast
-#elif defined(__sparc)
-	DynLoader::fptr rfptr = (DynLoader::fptr) dlsym(handle, name);	//lint !e611 //: Suspicious cast
-#elif defined(__OS2__)
+#if defined(__linux) || defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__hpux) || defined(__sparc) || defined(__OS2__) || defined(_AIX) || (defined (HAVE_DLFCN_H) && (HAVE_DLFCN_H==1 ) ) 
 	DynLoader::fptr rfptr = (DynLoader::fptr) dlsym(handle, name);	//lint !e611 //: Suspicious cast
 #elif defined(_WIN32)
 	DynLoader::fptr rfptr = (DynLoader::fptr) GetProcAddress((HINSTANCE) handle, name);	//lint !e611 //: Suspicious cast
 #else
-	system unsupported so far
+	#error "system unsupported so far"
 #endif
 	if ((rfptr == 0) && check) {
 		cerr << "error during getSymbol for " << name << ":" << dlerror()
@@ -203,6 +173,8 @@ DynLoader::fptr DynLoader::getSymbol(const char *name, int check) const
 
 // now the stuff handling search for plugins and loading
 
+//lint -sem(PluginVector::add, custodial(1))
+
 // OK, I could take a vector<DynLoader *> but that is not supported with all compilers yet.
 class PluginVector {
 public:
@@ -230,12 +202,27 @@ static PluginVector LoadedPlugins;
 
 static void loadaPlugin(const char *filename, ostream & errstream,bool verbose)
 {
+	if (verbose && filename) errstream << "loading plugin: " << filename << endl;
+
 	DriverDescription::currentfilename = filename;
 	DynLoader *dynloader = new DynLoader(filename,verbose);
 	if (!dynloader->valid()) {
 		delete dynloader;
 		errstream << "Problem during opening of pstoedit driver plugin: " << filename << ". This is no problem as long the driver in this library is not needed. Possibly you need to install further libraries and/or extend the LD_LIBRARY_PATH (*nix) or PATH (Windows) environment variables." << endl;
 		return;
+	} else {
+#if defined (__APPLE__)
+// on MacOS it seems necessary to at least do a dlsym to an existing symbol 
+// to get the global ctors beeing called
+		void (*fp)() =  ( ( void (*)() ) dynloader->getSymbol("initlibrary") );  
+		if (fp) {
+			if (verbose) {
+				errstream << "getSymbol(\"initlibrary\") succeeded." << endl;
+			}
+		} else {
+				errstream << "getSymbol(\"initlibrary\") failed." << endl;
+		}
+#endif
 	}
 	LoadedPlugins.add(dynloader);
 
@@ -264,51 +251,68 @@ static void loadaPlugin(const char *filename, ostream & errstream,bool verbose)
 //          globalRp->explainformats(errstream);
 }
 
-
-#if defined(__linux) || defined(__linux__)  || defined(__FreeBSD__) || defined(__sparc) || defined(__hpux) || defined(__OS2__)
 // for directory search
-#if HAVE_DIRENT_H
+#if (defined (HAVE_DLFCN_H) && (HAVE_DLFCN_H==1 ) ) 
 #include <dirent.h>
-#elif HAVE_SYS_NDIR_H
+#define DIR_VERSION 1
+#elif (defined (HAVE_SYS_NDIR_H) && (HAVE_SYS_NDIR_H==1 ) ) 
 #include <sys/ndir.h>
-#elif HAVE_SYS_DIR_H
+#define DIR_VERSION 2
+#elif (defined (HAVE_SYS_DIR_H) && (HAVE_SYS_DIR_H==1 ) ) 
 #include <sys/dir.h>
-#elif HAVE_NDIR_H
+#define DIR_VERSION 3
+#elif (defined (HAVE_NDIR_H) && (HAVE_NDIR_H==1 ) ) 
 #include <ndir.h>
+#define DIR_VERSION 4
 #else
+//
 // last chance
+#if defined(__linux) || defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__hpux) || defined(__sparc) || defined(__OS2__) || defined(_AIX) 
 #include <direct.h>
+#define DIR_VERSION 5
+#else
+// if not even direct.h is available we leave DIR_VERSION unset and later skip to Windows
 #endif
 
+#endif
+
+#ifdef DIR_VERSION
+// this is for all *nix like systems
 void loadPlugInDrivers(const char *pluginDir, ostream & errstream,bool verbose)
 {
 	DIR *dirp;
 	struct dirent *direntp;
+#if defined(__OS2__) || defined(__CYGWIN32__)
+	const char * const suffix = ".dll";
+#elif defined (__APPLE__)
+	const char * const suffix = ".dylib";
+#elif defined (__hpux)
+	const char * const suffix = ".sl";
+#else
+	const char * const suffix = ".so";
+#endif
+	if (verbose && pluginDir )  errstream << "loading plugins from " << pluginDir << " using suffix: " << suffix << endl;
 
 	dirp = opendir(pluginDir);
-	if (!dirp)
+	if (!dirp) {
+		if (verbose && pluginDir )  errstream << "opendir failed on: " << pluginDir << endl;
 		return;
+	}
+
 	while ((direntp = readdir(dirp)) != NIL) {
 //      cout <<  direntp->d_name << endl;
 		unsigned int flen = strlen(direntp->d_name);
-#ifdef __OS2__
-		char *expectedpositionofsuffix = direntp->d_name + flen - 4;
-#else
-		char *expectedpositionofsuffix = direntp->d_name + flen - 3;
-#endif
+		char *expectedpositionofsuffix = direntp->d_name + flen - strlen(suffix);
 //              if ( local filename starts with drv or plugins and ends with .so)
-		if (((strstr(direntp->d_name, "libp2edrv") == direntp->d_name) ||
+#if defined (__CYGWIN32__)
+		const char * const libprefix = "cygp2edrv";
+#else
+		const char * const libprefix = "libp2edrv";
+#endif
+		if (((strstr(direntp->d_name, libprefix) == direntp->d_name) ||
 			 (strstr(direntp->d_name, "plugin") == direntp->d_name)
 			) &&
-#ifdef __OS2__
-			(strstr(expectedpositionofsuffix, ".dll") == expectedpositionofsuffix)) {
-#else
-	#ifdef __hpux
-			(strstr(expectedpositionofsuffix, ".sl") == expectedpositionofsuffix)) {
-	#else
-			(strstr(expectedpositionofsuffix, ".so") == expectedpositionofsuffix)) {
-	#endif
-#endif
+			(strstr(expectedpositionofsuffix, suffix) == expectedpositionofsuffix)) {
 			char *fullname = new char[strlen(pluginDir) + flen + 2];
 			strcpy(fullname, pluginDir);
 			strcat(fullname, "/");
@@ -328,31 +332,37 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream,bool verbose)
 {
 	char szExePath[1000];
 	szExePath[0] = '\0';
-	P_GetPathToMyself("pstoedit", szExePath, sizeof(szExePath));
+	(void) P_GetPathToMyself("pstoedit", szExePath, sizeof(szExePath));
 
 	WIN32_FIND_DATA finddata;
 //   = getPluginDirectory(errstream);
 	if (pluginDir != 0) {
-		char pattern[] = "/*.dll";
+		const char pattern[] = "/*.dll";
 		char *searchpattern = new char[strlen(pluginDir) + strlen(pattern) + 1];
 		strcpy(searchpattern, pluginDir);
 		strcat(searchpattern, pattern);
 		HANDLE findHandle = FindFirstFile(searchpattern, &finddata);
 		if (findHandle == INVALID_HANDLE_VALUE) {
-			errstream << "Could not open plug-in directory (" << pluginDir
+			if (verbose) errstream << "Could not open plug-in directory (" << pluginDir
 				<< " or didn't find any plugin there" << endl;
 		} else {
 			BOOL more = FALSE;
 			do {
-
-				char *fullname = new char[strlen(pluginDir) + strlen(finddata.cFileName) + 3];
+#ifdef xx
+#define stricmp _stricmp
+#endif
+				// check for suffix beeing really .dll because FindFirstFile also matches
+				// files such as e.g. .dllx
+				const unsigned int len = strlen(finddata.cFileName);
+				// -4 means go back the length of ".dll"
+				if (stricmp(&finddata.cFileName[len -4 ],".dll") == 0) {
+				// cout << &finddata.cFileName[len -4 ] << endl;
+				char *fullname = new char[strlen(pluginDir) + len + 3];
 				strcpy(fullname, pluginDir);
 				strcat(fullname, "\\");
 				strcat(fullname, finddata.cFileName);
 //              errstream << "szExePath " << szExePath << endl;
-#ifdef xx
-#define stricmp _stricmp
-#endif
+
 				if ( (stricmp(fullname, szExePath) != 0) && (stricmp(finddata.cFileName, "pstoedit.dll") !=0 ) ) {
 					// avoid loading dll itself again
 					//                 errstream << "loading " << fullname << endl;
@@ -360,16 +370,17 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream,bool verbose)
 				} else {
 					//                 errstream << "ignoring myself " << finddata.cFileName << endl;
 				}
-				delete[]fullname;
+				delete[] fullname;
+				}
 				more = FindNextFile(findHandle, &finddata);
 			} while (more);
-			FindClose(findHandle);
+			(void) FindClose(findHandle);
 		}
 		delete[]searchpattern;
 	}
 }
 #else
-#error "system unsupported so far"
+#error "system unsupported so far (directory reading)"
 #endif
 
 #endif 
@@ -386,6 +397,7 @@ class ostream;
 #endif
 void loadPlugInDrivers(const char *pluginDir, ostream & errstream,bool verbose)
 {
+	if (verbose) { errstream << "dummy version of loadPlugInDrivers called since you system doesn't seem to support loading libraries at runtime - If you think that this is not correct, contact the author of this program (wglunz34_AT_pstoedit.net) " << endl; }
 }								// just a dummy Version
 #endif
  
